use anyhow::anyhow;
use clap::arg;
use prymn_agent::{self_update, server};

fn main() -> anyhow::Result<()> {
    if std::env::consts::OS != "linux" {
        return Err(anyhow!("platform is not linux!"));
    }

    tracing_subscriber::fmt().init();

    let command = clap::Command::new(env!("CARGO_BIN_NAME"))
        .version(env!("CARGO_PKG_VERSION"))
        .arg(arg!(-d --daemon "Run agent as daemon").exclusive(true))
        .arg(arg!(--install "Install this agent binary to the system").exclusive(true))
        .arg(arg!(--update "Download updated agent version and install it").exclusive(true))
        .arg_required_else_help(true)
        .try_get_matches()
        .unwrap_or_else(|e| e.exit());

    let is_daemon = command.get_flag("daemon");
    let is_install = command.get_flag("install");
    let is_update = command.get_flag("update");

    if is_daemon {
        tracing::info!("running as daemon");
        server::main()
    } else if is_install {
        tracing::info!("running install mode");
        self_update::install()
    } else if is_update {
        tracing::info!("running update mode");
        self_update::update()
    } else {
        unreachable!()
    }
}
